<?php
/*
 * Copyright (c) 2018-2021 Atipay, (https://www.atipay.net)

 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 * */

namespace PayWithAtipay;


/**
 * Atipay Payments Gateway
 *
 * @package     EDD
 * @subpackage  Gateways
 * @copyright   Copyright (c) 2021, Atipay
 * @since       1.0
 */

// Exit if accessed directly
if (!defined('ABSPATH')) exit;

final class EDD_Atipay_Payments
{

    private static $instance;
    public $gateway_id = 'atipay';
    private $refId;
    private $is_setup;


    /**
     * Get things going
     *
     * @access private
     * @since  1.0
     * @return void
     */
    private function __construct()
    {
        // Run this separate so we can ditch as early as possible
        $this->register();

        if (!edd_is_gateway_active($this->gateway_id)) {
            return;
        }

        $this->config();
        $this->includes();
        $this->filters();
        $this->actions();

    }

    /**
     * Register the payment gateway
     *
     * @access private
     * @since  1.0
     * @return void
     */
    private function register()
    {

        add_filter('edd_payment_gateways', array($this, 'register_gateway'), 1, 1);
    }

    /**
     * Setup constant configuration for file paths
     *
     * @access private
     * @since  1.0
     * @return void
     */
    private function config()
    {

        if (!defined('EDD_ATIPAY_CLASS_DIR')) {
            $path = __DIR__ . '/libs/atipay';
            define('EDD_ATIPAY_CLASS_DIR', trailingslashit($path));
        }
    }

    /**
     * Load additional files
     *
     * @access private
     * @since  1.0
     * @return void
     */
    private function includes()
    {

        // Include the Atipay Library
        require_once EDD_ATIPAY_CLASS_DIR . 'fn.atipay.php'; // Requires the other files itself
    }

    /**
     * Add filters
     *
     * @since  1.0
     * @return void
     */
    private function filters()
    {

        add_filter('edd_accepted_payment_icons', array($this, 'register_payment_icon'), 10, 1);
        add_filter('edd_show_gateways', array($this, 'maybe_hide_gateway_select'));


        if (is_admin()) {
            add_filter('edd_settings_sections_gateways', array($this, 'register_gateway_section'), 1, 1);
            add_filter('edd_settings_gateways', array($this, 'register_gateway_settings'), 1, 1);
        }
    }

    /**
     * Add actions
     *
     * @access private
     * @since  1.0
     * @return void
     */
    private function actions()
    {

        add_action('edd_atipay_cc_form', array($this, 'cc_form'), 1);
        add_action('edd_pre_process_purchase', array($this, 'check_config'), 2);
        add_action('edd_gateway_atipay', array($this, 'process_purchase'));
        add_action('edd_verify_atipay', array($this, 'verify_purchase'));
        add_action('init', array($this, 'check_psp_return'));

    }

    /**
     * Retrieve current instance
     *
     * @access private
     * @since  1.0
     * @return EDD_Atipay_Payments instance
     */
    public static function getInstance()
    {
        if (!isset(self::$instance) && !(self::$instance instanceof EDD_Atipay_Payments)) {
            self::$instance = new EDD_Atipay_Payments;
        }
        return self::$instance;
    }

    /**
     * Show an error message on checkout if Atipay is enabled but not setup.
     *
     * @since 1.0
     */
    public function check_config()
    {
        $is_enabled = edd_is_gateway_active($this->gateway_id);
        if ((!$is_enabled ||
                $this->is_setup() === false
            )
            &&
            edd_get_chosen_gateway() == 'atipay') {

            edd_set_error('atipay_gateway_not_configured', __('There is an error with the Atipay Payments configuration.', 'easy-digital-downloads'));
        }
    }

    /**
     * Method to check if all the required settings have been filled out, allowing us to not output information without it.
     *
     * @since 1.0
     * @return bool
     */
    public function is_setup()
    {

        if (null !== $this->is_setup) {
            return $this->is_setup;
        }

        $required_items = array('atipay_api_key', 'atipay_currency');

        $current_values = array(
            'atipay_api_key' => edd_get_option('atipay_api_key', ''),
            'atipay_currency' => edd_get_option('atipay_currency', ''),
        );


        $this->is_setup = true;

        foreach ($required_items as $key) {
            if (empty($current_values[$key])) {
                $this->is_setup = false;
                break;
            }
        }

        return $this->is_setup;
    }

    /**
     * Register the gateway
     *
     * @since  1.0
     * @param  $gateways array
     * @return array
     */
    public function register_gateway($gateways)
    {

        $default_atipay_info = array(
            $this->gateway_id => array(
                'admin_label' => __('شبکه همراه آتی‌پی', 'easy-digital-downloads'),
                'checkout_label' => __('شبکه همراه آتی‌پی', 'easy-digital-downloads'),
                'supports' => array(),
            ),
        );

        $default_atipay_info = apply_filters('edd_register_atipay_gateway', $default_atipay_info);
        $gateways = array_merge($gateways, $default_atipay_info);

        return $gateways;

    }

    /**
     * Register the payment icon
     *
     * @since  1.0
     * @param  array $payment_icons Array of payment icons
     * @return array                The array of icons with Atipay Added
     */
    public function register_payment_icon($payment_icons)
    {
        return;
    }

    /**
     * Hides payment gateway select options after return from Atipay
     *
     * @since  2.7.6
     * @param  bool $show Should gateway select be shown
     * @return bool
     */
    public function maybe_hide_gateway_select($show)
    {

        return false;

    }

    /**
     * Register the payment gateways setting section
     *
     * @since  2.5
     * @param  array $gateway_sections Array of sections for the gateways tab
     * @return array                   Added Atipay Payments into sub-sections
     */
    public function register_gateway_section($gateway_sections)
    {
        $gateway_sections['atipay'] = 'درگاه شبکه همراه آتی‌پی';

        return $gateway_sections;
    }

    /**
     * Register the gateway settings
     *
     * @since  1.0
     * @param  $gateway_settings array
     * @return array
     */
    public function register_gateway_settings($gateway_settings)
    {

        $default_atipay_settings = array(
            'atipay' => array(
                'id' => 'atipay',
                'name' => '<strong>تنظیمات </strong>',
                'type' => 'header',
            ),
            'atipay_register' => array(
                'id' => 'atipay_register',
                'name' => 'ثبت نام در شبکه همراه آتی‌پی',
                'desc' => '<p><a href="https://www.atipay.net" class="button" target="_blank">' .
                    'شبکه همراه آتی‌پی' .
                    '</a></p>',
                'type' => 'descriptive_text',
            ),
            'atipay_api_key' => array(
                'id' => 'atipay_api_key',
                'name' => 'ApiKey',
                'desc' => 'برای شما از طریق آتی‌پی ارسال می‌شود.',
                'type' => 'text',
                'size' => 'regular',
            ),
            'atipay_currency' => array(
                'id' => 'atipay_currency',
                'name' => 'نوع ارز',
                'type' => 'radio',
                'options' => array('R' => 'ریال', 'T' => 'تومان'),
                'std' => 'ir',
                'desc' => 'نوع ارز در فروشگاه شما'
            ),
        );

        $default_atipay_settings = apply_filters('edd_default_atipay_settings', $default_atipay_settings);
        $gateway_settings['atipay'] = $default_atipay_settings;

        return $gateway_settings;
    }


    /**
     * Process the purchase and create the charge in Atipay
     *
     * @since  1.0
     * @param  $purchase_data array Cart details
     * @return void
     */
    public function process_purchase($purchase_data)
    {
        $errors = edd_get_errors();
        if ($errors) {
            edd_send_back_to_checkout('?payment-mode=atipay');
        }

        // Setup payment data to be recorded
        $payment_data = array(
            'price' => $purchase_data['price'],
            'date' => $purchase_data['date'],
            'user_email' => $purchase_data['user_email'],
            'purchase_key' => $purchase_data['purchase_key'],
            'currency' => edd_get_currency(),
            'downloads' => $purchase_data['downloads'],
            'user_info' => $purchase_data['user_info'],
            'cart_details' => $purchase_data['cart_details'],
            'gateway' => $this->gateway_id,
            'status' => 'pending',
        );

        $payment_id = edd_insert_payment($payment_data);


        $callback = edd_get_checkout_uri(array('payment_id' => $payment_id));
        $amount = intval($purchase_data['price']);

        $currency = edd_get_option('atipay_currency');
        $token_amount = $amount;
        if ($currency == 'T') {
            $token_amount = $amount * 10;
        }
        $token_params = array(
            'apiKey' => edd_get_option('atipay_api_key'),
            'redirectUrl' => $callback,
            'invoiceNumber' => $payment_id,
            'amount' => $token_amount,
            'description' => 'PaymentID #' . $payment_id,
        );

        $r = fn_atipay_get_token($token_params);
        if ($r['success'] == 1) {
            $form = fn_atipay_redirect_to_psp_form($r['token'], 'atipay_edd_form');
            echo $form;
        } else {
            $error_message = $r['errorMessage'];
            edd_set_error('atipay_edd_token_error', $error_message);
            edd_send_back_to_checkout('?payment-mode=atipay');
        }


    }

    public function verify_purchase()
    {
        $params = $_POST;
        $result = fn_check_callback_data($params);
        $payment_id = $params['reservationNumber'];
        if ($result['success'] == 1) { //will verify here
            $apiKey = edd_get_option('atipay_api_key');
            $purchase = edd_get_payment($payment_id);
            $amount = intval($purchase->total);
            $currency = edd_get_option('atipay_currency');
            if ($currency == 'T') {
                $amount = $amount * 10;
            }
            $verify_params = array('apiKey' => $apiKey,
                'referenceNumber' => $params['referenceNumber']
            );
            $r = fn_atipay_verify_payment($verify_params, $amount);
            if ($r['success'] == 0) { //veriy failed
                $error_message = $r['errorMessage'];
                edd_update_payment_status($payment_id, 'failed');
                edd_set_error('atipay_edd_psp_back_error_1', $error_message);
                edd_send_back_to_checkout('?payment-mode=atipay');
            } else { //success
                edd_update_payment_meta($payment_id, '_edd_payment_ref_id', $params['referenceNumber']);
                edd_update_payment_meta($payment_id, '_edd_payment_rrn', $params['rrn']);
                edd_update_payment_meta($payment_id, '_edd_payment_trace_number', $params['traceNumber']);
                edd_set_payment_transaction_id($payment_id, $params['referenceNumber']);
                edd_update_payment_status($payment_id, 'publish');
                edd_empty_cart();
                edd_send_to_success_page();
            }
        } else {
            edd_update_payment_status($payment_id, 'failed');
            $error_message = $result['error'];
            edd_set_error('atipay_edd_psp_back_error_1', $error_message);
            edd_send_back_to_checkout('?payment-mode=atipay');

        }

    }

    function cc_form()
    {
        return;
    }

    function check_psp_return()
    {

        if ($_SERVER['REQUEST_METHOD'] == 'POST') {
            if (isset($_POST['reservationNumber']) && !empty($_POST['reservationNumber'])) {
                $reservationNumber = $_POST['reservationNumber'];
                if (isset($_GET['payment_id']) && !empty($_GET['payment_id'])) {
                    $payment_id = $_GET['payment_id'];
                    if ($reservationNumber != $payment_id) {
                        $error_message = "خطا در اطلاعات ارسالی از درگاه پرداخت";
                        edd_set_error('atipay_edd_psp_return_error_1', $error_message);
                        edd_send_back_to_checkout('?payment-mode=atipay');
                        return;
                    } else {
                        do_action('edd_verify_atipay');
                    }
                } else {
                    die('Wrong Data');
                }
            }
        }


    }

    /**
     * Retrieve the client object
     *
     * @access private
     * @since  1.0
     * @return PayWithAtipay\Client
     */
    private function get_client()
    {

        if (!$this->is_setup()) {
            return false;
        }

        if (!is_null($this->client)) {
            return $this->client;
        }

        $this->setup_client();

        return $this->client;
    }

    private function get_registration_url()
    {
        $base_url = 'https://payments.atipay.com/register';

        $query_args = array(
            'registration_source' => 'SPPD',
            'spId' => 'A3JST9YM1SX7LB',
        );

        return add_query_arg($query_args, $base_url);
    }

}

/**
 * Load EDD_Atipay_Payments
 *
 * @since  1.0
 * @return object EDD_Atipay_Payments
 */
function EDD_Atipay()
{
    return EDD_Atipay_Payments::getInstance();
}

EDD_Atipay();
?>